<?php
if( !defined( 'ABSPATH' ) ) exit; // Exit if accessed directly.
/**
 *
 * @class   ag_epdq_sub_checkout
 * @extends ag_epdq_checkout
 * @version 1.0.0
 */
class ag_epdq_sub_checkout extends ag_epdq_checkout {
  /**
   * Constructor
   */
  public function __construct() {
    parent::__construct();
    if( class_exists( 'WC_Subscriptions_Order' ) ) {
      add_action( 'scheduled_subscription_payment_' . $this->id, array( $this, 'scheduled_subscription_payment' ), 10, 3 );
    }
  }
  /**
   * Process the subscription
   *
   * @param  int $order_id
   * @return array
   */
  public function process_subscription( $order_id ) {
    $order = new WC_Order( $order_id );
    if( $this->sandbox == 'yes' ) {
      $error_msg .= ' ' . __( 'Developers: Please make sure that you\'re including jQuery and there are no JavaScript errors on the page.', 'ag-epdq-subs' );
    }
    $initial_payment = WC_Subscriptions_Order::get_total_initial_payment( $order );
    if( $initial_payment > 0 ) {
      $payment_response = $this->process_subscription_payment( $order, $initial_payment );
    }
    if( isset( $payment_response ) && is_wp_error( $payment_response ) ) {
      throw new Exception( $payment_response->get_error_message() );
    }
    else {
      // Remove cart
      WC()->cart->empty_cart();
      // Return thank you page redirect
      return array(
        'result'   => 'success',
        'redirect' => $this->get_return_url( $order )
      );
    }
    return array(
      'result'   => 'fail',
      'redirect' => ''
    );
  }
  /**
   * Process the pre-order
   *
   * @param  int $order_id
   * @return array
   */
  public function process_pre_order( $order_id ) {
    if( WC_Pre_Orders_Order::order_requires_payment_tokenization( $order_id ) ) {
      $order = new WC_Order( $order_id );
      // Order limit
      if( $order->order_total * 100 < 50 ) {
        $error_msg = __( 'Sorry, the minimum allowed order total is 0.50 to use this payment method.', 'ag-epdq-subs' );
      }
      if( $this->sandbox == 'yes' ) {
        $error_msg .= ' ' . __( 'Developers: Please make sure that you\'re including jQuery and there are no JavaScript errors on the page.', 'ag-epdq-subs' );
      }
      // Reduce stock levels
      $order->reduce_order_stock();
      // Remove cart
      WC()->cart->empty_cart();
      // Is pre ordered!
      WC_Pre_Orders_Order::mark_order_as_pre_ordered( $order );
      // Return thank you page redirect
      return array(
        'result'   => 'success',
        'redirect' => $this->get_return_url( $order )
      );
      return array(
        'result'   => 'fail',
        'redirect' => ''
      );
    }
    else {
      return parent::process_payment( $order_id );
    }
  }
  /**
   * Process the payment
   *
   * @param  int $order_id
   * @return array
   */
  public function process_payment( $order_id ) {
    // Processing subscription
    if( class_exists( 'WC_Subscriptions_Order' ) && WC_Subscriptions_Order::order_contains_subscription( $order_id ) ) {
      return $this->process_subscription( $order_id );
    // Processing pre-order
    }
    else {
      return parent::process_payment( $order_id );
    }
  }
  /**
   * process_subscription_payment function.
   *
   * @param  WC_order $order
   * @param  integer $amount (default: 0)
   * @return bool|WP_Error
   */
  public function process_subscription_payment( $order, $amounts ) {
    $order_items       = $order->get_items();
    $order_item        = array_shift( $order_items );
    $subscription_name = sprintf( __( '%s - Subscription for "%s"', 'ag-epdq-subs' ), esc_html( get_bloginfo( 'name' ) ), $order_item['name'] ) . ' ' . sprintf( __( '(Order %s)', 'ag-epdq-subs' ), $order->get_order_number() );
    //if( $amount * 100 < 50 ) {
    //  return new WP_Error( 'simplify_error', __( 'Sorry, the minimum allowed order total is 0.50 to use this payment method.', 'ag-epdq-subs' ) );
    //}

    global $woocommerce;

		// Get this Order's information so that we know
		// who to charge and how much
		//$order = new WC_Order( $order_id );

		// Are we testing right now or is it a real transaction
		$environment = ( $this->environment == "yes" ) ? 'TRUE' : 'FALSE';

		// Decide which URL to post to
		if( is_ssl()) {
			$environment_url = ( "FALSE" == $environment )
							   ? 'https://payments.epdq.co.uk/ncol/prod/orderdirect.asp'
							   : 'https://mdepayments.epdq.co.uk/ncol/test/orderdirect.asp';
		} else {
			$environment_url = 'https://mdepayments.epdq.co.uk/ncol/test/orderdirect.asp';
		}


		// This is where the fun stuff begins
    $Passphrase = $this->sha_in; // your SHA-IN pass phrase goes here
    $data_string = '';

    foreach ($data as $key => $value) {
      $data_string[] = $key. '='.$value.$Passphrase; // creates the string to hash
    }
    $string_post = '';
    $string_post = implode ('', $data_string);
    $SHASign = sha1 ($string_post);

		// sorting field
		$fullname = $order->billing_first_name . ' - ' . $order->billing_last_name;
		$amounttocharge = floatval( preg_replace( '#[^\d.]#', '', $order->order_total*100 ) );
		$order_item = $order->get_items();
		foreach( $order_item as $product ) {
		  $prodct_name[] = $product['name']." x".$product['qty'];
		}
		$product_list = implode( ',', $prodct_name );

    //construct the array again and post the SHASign as a name value pair
    $data_post;
    $data_post['PSPID'] = $this->api_login;
    $data_post['ORDERID'] = $order->get_order_number();
    $data_post['USERID'] = $this->api_user;
    $data_post['PSWD'] = $this->api_password;
    $data_post['AMOUNT'] = $amounttocharge;
    $data_post['CURRENCY'] = get_woocommerce_currency();
    //$data_post['CARDNO'] = str_replace( array(' ', '-' ), '', $_POST['ag_epdq_checkout-card-number'] );
    //$data_post['ED'] = str_replace( array( '/', ' '), '', $_POST['ag_epdq_checkout-card-expiry'] );
    //$data_post['CVC'] = ( isset( $_POST['ag_epdq_checkout-card-cvc'] ) ) ? $_POST['ag_epdq_checkout-card-cvc'] : '';
    $data_post['SHASign'] = $SHASign;
		//$data_post['EMAIL'] = $order->billing_email;
		//$data_post['CN'] = $fullname;
		$data_post['COM'] = $product_list;
		//$data_post['OWNERADDRESS'] = $order->billing_address_1;
		//$data_post['OWNERZIP'] = $order->billing_postcode;
		//$data_post['OWNERCTY'] = $order->billing_city;
		//$data_post['OWNERTELNO'] = $order->billing_phone;
		$data_post['REMOTE_ADDR'] = $_SERVER['REMOTE_ADDR'];
		// 3D secure bits
		$data_post['WIN3DS'] = $this->secure_3d;
		$data_post['FLAG3D'] = 'Y';
		$data_post['ACCEPTURL'] = $this->notify_url;
		$data_post['DECLINEURL'] = $this->notify_url;
		$data_post['EXCEPTIONURL'] = $this->notify_url;
		$data_post['LANGUAGE'] = get_bloginfo('language');
		$data_post['OPERATION'] = $this->operation;
    $data_post['ALIAS'] = 'testingthis33';
    //$data_post['ALIASUSAGE'] = 'Adding alias to epdq223';

		// If we are in sub mode add these two fields to add alias
		//if ( WC_Subscriptions_Cart::cart_contains_subscription() ) {

    //}


    $post_string = '';

    foreach ($data_post as $key => $value) {
      $post_string[] = $key. '=' .$value;
    }

    $actual_string = '';
    $actual_string = implode ('&', $post_string);
		$epdq_connection = curl_init($environment_url);

		curl_setopt ($epdq_connection, CURLOPT_CONNECTTIMEOUT, 20);
		curl_setopt ($epdq_connection, CURLOPT_RETURNTRANSFER, true);
		curl_setopt ($epdq_connection, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($epdq_connection,CURLOPT_USERAGENT,'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13');
		curl_setopt ($epdq_connection, CURLOPT_POSTFIELDS, $actual_string);

		$result = curl_exec($epdq_connection);
		$http_code = curl_getinfo($epdq_connection, CURLINFO_HTTP_CODE);
		$xmldata = XMLtoArray($result);
		//$this->transaction_successfull($xmldata);
		curl_close($epdq_connection);

		// Test the code to know if the transaction went through or not.
		if ( ( $xmldata['NCRESPONSE']['STATUS'] == 5 ) || ( $xmldata['NCRESPONSE']['STATUS'] == 9 ) ) {

			if (!empty($xmldata['NCRESPONSE']['ORDERID']))
					$note = 'Order ID: ' . $xmldata['NCRESPONSE']['ORDERID'] . '.<br>'; //	order id
			if (!empty($xmldata['NCRESPONSE']['AMOUNT']))
					$note .= 'Amount: ' . $xmldata['NCRESPONSE']['AMOUNT'] . '.<br>'; //	amount
			if (!empty($xmldata['NCRESPONSE']['CURRENCY']))
					$note .= 'Order currency: ' . $xmldata['NCRESPONSE']['CURRENCY'] . '.<br>'; //	order currency
			if (!empty($xmldata['NCRESPONSE']['PM']))
					$note .= 'Payment Method: ' . $xmldata['NCRESPONSE']['PM'] . '.<br>'; //	payment method
			if (!empty($xmldata['NCRESPONSE']['ACCEPTANCE']))
					$note .= 'Acceptance code returned by acquirer: ' . $xmldata['NCRESPONSE']['ACCEPTANCE'] . '.<br>';    //	acceptance
			if (!empty($xmldata['NCRESPONSE']['STATUS']))
					$note .= 'Transaction status : ' . get_epdq_direct_status_code($xmldata['NCRESPONSE']['STATUS']) . '.<br>'; //	status code
			//if (!empty($args['CARDNO']))
			//		$note .= 'Masked card number : ' . $CARDNO . '.<br>'; //	catd no
			if (!empty($xmldata['NCRESPONSE']['PAYID']))
					$note .= 'Payment reference in ePDQ system: ' . $xmldata['NCRESPONSE']['PAYID'] . '.<br>'; //	pay id
			if (!empty($xmldata['NCRESPONSE']['NCERROR']))
					$note .= 'Error Code: ' . $xmldata['NCRESPONSE']['NCERROR'] . '.<br>'; //	ncerror
			if (!empty($xmldata['NCRESPONSE']['BRAND']))
					$note .= 'Card brand (ePDQ system derives this from the card number) : ' . $xmldata['NCRESPONSE']['BRAND'] . '.<br>'; //	brand
			//if (!empty($args['ED']))
					//$note .= 'Payer\'s card expiry date : ' . $ED . '.<br>'; //	expiry date
			if (!empty($xmldata['NCRESPONSE']['TRXDATE']))
					$note .= 'Transaction Date: ' . $xmldata['NCRESPONSE']['TRXDATE'] . '.<br>'; //	date
			if (!empty($xmldata['NCRESPONSE']['CN']))
					$note .= 'Cardholder/customer name: ' . $xmldata['NCRESPONSE']['CN'] . '.<br>'; //	payer's name
			if (!empty($xmldata['NCRESPONSE']['IP']))
					$note .= 'Customer\'s IP: ' . $xmldata['NCRESPONSE']['IP'] . '.<br>'; //	payer's ip

			if (!empty($xmldata['NCRESPONSE']['AAVADDRESS']))
					$note .= 'AAV result for the address: ' . $xmldata['NCRESPONSE']['AAVADDRESS'] . ' . <br>'; //	aav address
			if (!empty($xmldata['NCRESPONSE']['AAVCHECK']))
					$note .= 'Result of the automatic address verification: ' . $xmldata['NCRESPONSE']['AAVCHECK'] . ' . <br>'; //	aav check
			if (!empty($xmldata['NCRESPONSE']['AAVZIP']))
					$note .= 'AAV result for the zip code: ' . $xmldata['NCRESPONSE']['AAVZIP'] . ' . <br>'; // aav zip
			//if (!empty($args['BIN']))
					//$note .= 'First 6 digits of credit card number: ' . $BIN . ' . <br>'; // bin
			if (!empty($xmldata['NCRESPONSE']['CCCTY']))
					$note .= 'Country where the card was issued: ' . $xmldata['NCRESPONSE']['CCCTY'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['COMPLUS']))
					$note .= 'Custom value passed: ' . $xmldata['NCRESPONSE']['COMPLUS'] . ' . <br>';

			if (!empty($xmldata['NCRESPONSE']['CVCCHECK']))
					$note .= 'Result of the card verification code check: ' . $xmldata['NCRESPONSE']['CVCCHECK'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['ECI']))
					$note .= 'Electronic Commerce Indicator: ' . $xmldata['NCRESPONSE']['ECI'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['FXAMOUNT']))
					$note .= 'FXAMOUNT: ' . $xmldata['NCRESPONSE']['FXAMOUNT'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['FXCURRENCY']))
					$note .= 'FXCURRENCY: ' . $xmldata['NCRESPONSE']['FXCURRENCY'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['IPCTY']))
					$note .= 'Originating country of the IP address: ' . $xmldata['NCRESPONSE']['IPCTY'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['SUBBRAND']))
					$note .= 'SUBBRAND: ' . $xmldata['NCRESPONSE']['SUBBRAND'] . ' . <br>';
			if (!empty($xmldata['NCRESPONSE']['VC']))
					$note .= 'Virtual Card type: ' . $xmldata['NCRESPONSE']['VC'] . ' . <br>';

					// Store these for later
					if (isset($xmldata['NCRESPONSE']['ORDERID'])) {
						update_post_meta($order->id, 'OrderID', $xmldata['NCRESPONSE']['ORDERID']);
					}
					if (isset($xmldata['NCRESPONSE']['AMOUNT'])) {
						update_post_meta($order->id, 'OrderAmount', $xmldata['NCRESPONSE']['AMOUNT']);
					}
					if (isset($xmldata['NCRESPONSE']['CURRENCY'])) {
						update_post_meta($order->id, 'OrderCurrency', $xmldata['NCRESPONSE']['CURRENCY']);
					}
					if (isset($xmldata['NCRESPONSE']['PM'])) {
						update_post_meta($order->id, 'PaymentMethod', $xmldata['NCRESPONSE']['PM']);
					}
					if (isset($xmldata['NCRESPONSE']['ACCEPTANCE'])) {
						update_post_meta($order->id, 'Acceptance', $xmldata['NCRESPONSE']['ACCEPTANCE']);
					}
					if (isset($xmldata['NCRESPONSE']['STATUS'])) {
						update_post_meta($order->id, 'Status', get_epdq_direct_status_code($xmldata['NCRESPONSE']['STATUS']));
					}
					if (isset($xmldata['NCRESPONSE']['PAYID'])) {
						update_post_meta($order->id, 'PAYID', $xmldata['NCRESPONSE']['PAYID']);
					}
					if (isset($xmldata['NCRESPONSE']['NCERROR'])) {
						update_post_meta($order->id, 'NCERROR', $xmldata['NCRESPONSE']['NCERROR']);
					}
					if (isset($xmldata['NCRESPONSE']['BRAND'])) {
						update_post_meta($order->id, 'BRAND', $xmldata['NCRESPONSE']['BRAND']);
					}
					if (isset($xmldata['NCRESPONSE']['TRXDATE'])) {
						update_post_meta($order->id, 'TRXDATE', $xmldata['NCRESPONSE']['TRXDATE']);
					}
					if (isset($xmldata['NCRESPONSE']['IP'])) {
						update_post_meta($order->id, 'IP', $xmldata['NCRESPONSE']['IP']);
					}

          if (isset($xmldata['NCRESPONSE']['ALIAS'])) {
            update_post_meta($order->id, 'ALIAS', $xmldata['NCRESPONSE']['ALIAS']);
          }
          update_post_meta($order->id, 'Data back', $result);
          update_post_meta( $order->id, '_transaction_id', $xmldata['NCRESPONSE']['PAYID'] );

			// Payment has been successful
			$order->add_order_note( __( 'ePDQ payment completed.', 'ag_epdq_checkout' ) );

			// Mark order as Paid
			$order->payment_complete();

			// Empty the cart (Very important step)
			$woocommerce->cart->empty_cart();

			// Add status code & ePDQ order ID to order
			//$order->add_order_note( 'ePDQ order id: '. $xmldata['NCRESPONSE']['PAYID'] );
			//$order->add_order_note( 'Status Code: '. get_epdq_direct_status_code($xmldata['NCRESPONSE']['STATUS']) );
			$order->add_order_note($note);

			// Redirect to thank you page
			return array(
				'result'   => 'success',
				'redirect' => $this->get_return_url( $order ),
			);

		} elseif ( ( $xmldata['NCRESPONSE']['STATUS'] == 46 ) ) {

			WC()->session->set('HTML_ANSWER', $xmldata['NCRESPONSE']['HTML_ANSWER']);
			WC()->session->set('PaReq', $xmldata['NCRESPONSE']['PaReq']);
			WC()->session->set('Order_ID', $order_id);

			$redirect = $order->get_checkout_payment_url(true);

			return array(
					'result'    => 'success',
					'redirect'    => $redirect
			);


		} else {
			// Transaction was not succesful
      $order->add_order_note( __( 'Gateway name payment declined', 'woocommerce-payment-gateway-boilerplate' ) );
      return new WP_Error( 'gateway_name_payment_declined', __( 'Payment was declined - please try again.', 'woocommerce-payment-gateway-boilerplate' ) );

		}

  }
  /**
   * scheduled_subscription_payment function.
   *
   * @param  float $amount_to_charge The amount to charge.
   * @param  WC_Order $order The WC_Order object of the order which the subscription was purchased in.
   * @param  int $product_id The ID of the subscription product for which this payment relates.
   * @return void
   */
  public function scheduled_subscription_payment( $amount_to_charge, $order, $product_id ) {
    $result = $this->process_subscription_payment( $order, $amount_to_charge );
    if( is_wp_error( $result ) ) {
      WC_Subscriptions_Manager::process_subscription_payment_failure_on_order( $order, $product_id );
    }
    else {
      WC_Subscriptions_Manager::process_subscription_payments_on_order( $order );
    }
  }
}
?>
