<?php
# @Author: Aaron Bowie
# @Date:   Friday, June 16th 2017, 11:11:20 pm
# @Email:  support@weareag.co.uk
# @Filename: epdq.php
# @Last modified by:   Aaron Bowie
# @Last modified time: Wednesday, May 23rd 2018, 3:11:48 pm

/*
Plugin Name: AG Barclaycard ePDQ Direct Link Payment Gateway for WooCommerce
Plugin URI: https://www.weareag.co.uk/
Description: Extends WooCommerce by Adding the Barclaycard ePDQ Direct Link Payment Gateway.
Version: 1.1.4
Author: Aaron Bowie, We are AG
Author URI: https://www.weareag.co.uk/
WC requires at least: 3.0.0
WC tested up to: 3.4.0
*/

defined('ABSPATH') or die("No script kiddies please!");


// Plugin repo bits
$version = '1.1.4';
$plugin_id = '1c4s80ZjUSkh0i1i';
$repo_url = 'http://weareag.co.uk/update/';
// defined
if ( ! defined( 'AG_ePDQ_Repo' ) ) {
	define( 'AG_ePDQ_Repo', $repo_url );
}
if ( ! defined( 'plugin_id_ag' ) ) {
	define( 'plugin_id_ag', $plugin_id );
}
if ( ! defined( 'ag_PLUGIN_VER' ) ) {
	define( 'ag_PLUGIN_VER', $version );
}

if ( ! defined( 'AGD_KEY' ) ) {
	define( 'AGD_KEY', '56cdf3f59bdb87.92903929' );
}
if ( ! defined( 'AGD' ) ) {
	define( 'AGD', 'https://www.weareag.co.uk' );
}
if ( ! defined( 'ITEM' ) ) {
	define( 'ITEM', '2784' );
}

// Include our Gateway Class and register Payment Gateway with WooCommerce
add_action( 'plugins_loaded', 'ag_epdq_checkout_init', 0 );
function ag_epdq_checkout_init() {
	// If the parent WC_Payment_Gateway class doesn't exist
	// it means WooCommerce is not installed on the site
	// so do nothing
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) return;

	// If we made it this far, then include our Gateway Class
	include_once( 'epdq-class.php' );
	include_once( 'epdq-sub-class.php' );

	// Now that we have successfully included our class,
	// Lets add it too WooCommerce
	add_filter( 'woocommerce_payment_gateways', 'ag_epdq_checkout_gateway' );
	function ag_epdq_checkout_gateway( $methods ) {
		if ( class_exists( 'WC_Subscriptions_Order' ) ) {
			$methods[] = 'ag_epdq_sub_checkout';
		} else {
			$methods[] = 'ag_epdq_checkout';
		}
		return $methods;
	}
}


function rkv_load_update() {
	if ( ! class_exists( 'RKV_Remote_Updater' ) ) {
		include( 'lib/RKV_Remote_Updater.php' );
	}
}

add_action( 'admin_init', 'agd_ePDQ_remote_update' );
function agd_ePDQ_remote_update() {

	// ensure the class exists before running
	if ( ! class_exists( 'RKV_Remote_Updater' ) ) {
		return;
	}

	$updater = new RKV_Remote_Updater( AG_ePDQ_Repo, __FILE__, array(
		'unique'    => plugin_id_ag,
		'version'   => ag_PLUGIN_VER,
	)
	);
}


// Add custom action links
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'ag_epdq_checkout_action_links' );
function ag_epdq_checkout_action_links( $links ) {
	$plugin_links = array(
		'<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout' ) . '">' . __( 'Settings', 'ag_epdq_checkout' ) . '</a>',
	);
	// Merge our new link with the default ones
	return array_merge( $plugin_links, $links );
}


/*-----------------------------------------------------------------------------------*/
/*	Licence management section
/*-----------------------------------------------------------------------------------*/

if ( ! function_exists( 'ag_plugin_licence' ) ) {

	add_action('admin_menu', 'ag_plugin_licence');
	function ag_plugin_licence() {
	  add_menu_page('AG License Activation Menu', 'AG Plugins', 'manage_options', 'AG_licence', 'AG_plugins_page');
	}

}


if ( ! function_exists( 'AG_plugins_page' ) ) {

	function AG_plugins_page() {
	    echo '<div class="wrap">';
	    echo '<h2>We are AG Plugin License Management</h2>';
			echo '<p>Please enter the license key for this product to activate it. You were given a license key when you purchased this item.</p>';

			do_action( 'load_ag_plugins' );

			echo '<p>When moving plugin from one site to another (from development server to live site) you <strong>MUST</strong> deactivate your plugin first on development before you move it over.</p>';
	    echo '</div>';
	}

}

/*-----------------------------------------------------------------------------------*/
/*	End
/*-----------------------------------------------------------------------------------*/
add_action( 'load_ag_plugins', 'epdq_licence_settings' );
function epdq_licence_settings() {
	if (isset($_REQUEST['AGD_epdqapi_Key_activate_license'])) {
			$license_key = $_REQUEST['AGD_epdqapi_Key'];
			// API query parameters
			$api_params = array(
					'slm_action' => 'slm_activate',
					'secret_key' => AGD_KEY,
					'license_key' => $license_key,
					'registered_domain' => $_SERVER['SERVER_NAME'],
					'item_reference' => urlencode(pro),
			);
			// Send query to the license manager server
			$query = esc_url_raw(add_query_arg($api_params, AGD));
			$response = wp_remote_get($query, array('timeout' => 20, 'sslverify' => false));
			$license_data = json_decode(wp_remote_retrieve_body($response));

			if($license_data->result == 'success'){//Success was returned for the license activation
					update_option('AGD_epdqapi_Key', $license_key);
					update_option('AGD_epdqapi_Key_message', $license_data->message);
					add_option('AGD_epdqapi_Key_message', $license_data->message);
					$fulldate = $response['headers']['expires'];
					$words = explode(" ", $fulldate);
					$date = join(" ", array_slice($words, 0, 4));
					add_option('AGD_epdqapi_Key_expires', $date);
					update_option('AGD_epdqapi_Key_expires', $date);
			} else {
				update_option('AGD_epdqapi_Key_message', $license_data->message);
			}
	}
	/*** End of license activation ***/
	/*** License activate button was clicked ***/
	if (isset($_REQUEST['AGD_epdqapi_Key_deactivate_license'])) {
			$license_key = $_REQUEST['AGD_epdqapi_Key'];
			// API query parameters
			$api_params = array(
					'slm_action' => 'slm_deactivate',
					'secret_key' => AGD_KEY,
					'license_key' => $license_key,
					'registered_domain' => $_SERVER['SERVER_NAME'],
					'item_reference' => urlencode(pro),
			);
			$query = esc_url_raw(add_query_arg($api_params, AGD));
			$response = wp_remote_get($query, array('timeout' => 20, 'sslverify' => false));

			// License data.
			$license_data = json_decode(wp_remote_retrieve_body($response));
			// TODO - Do something with it.
			if($license_data->result == 'success'){//Success was returned for the license activation
					//Remove the licensse key from the options table. It will need to be activated again.
					update_option('AGD_epdqapi_Key', '');
					update_option('AGD_epdqapi_Key_message', $license_data->message);
					echo $response->headers->expires;
			} else {
				update_option('AGD_epdqapi_Key_message', $license_data->message);
			}
	}
?>
<form action="" method="post" style="padding-top: 20px; padding-bottom: 20px;">
	<table class="form-table">
			<tr>
				<label class="licence_label" for="AGD_epdqapi_Key"><strong>ePDQ Direct link Licence Key</strong></label>
<th style="width:10px; padding-bottom: 0;"><input class="regular-text" type="password" id="AGD_epdqapi_Key" name="AGD_epdqapi_Key"  value="<?php echo get_option('AGD_epdqapi_Key'); ?>" ><input type="submit" name="AGD_epdqapi_Key_activate_license" value="Activate" class="button-primary activate_license" /><input type="submit" name="AGD_epdqapi_Key_deactivate_license" value="Deactivate" class="button" />
<?php if (get_option('AGD_epdqapi_Key_message') == 'License key activated') { echo '<p style="color: green"><strong>'. get_option('AGD_epdqapi_Key_message'). '</strong></p>'; } elseif (get_option('AGD_epdqapi_Key_message') == 'Your License key has expired') {
echo '<p style="color: red"><strong>'. get_option('AGD_epdqapi_Key_message'). ' to continue getting support and updates make sure you <a href="">update your licence here</a> now.</strong></p>';
} else { echo '<p style="color: red"><strong>'. get_option('AGD_epdqapi_Key_message'). '</strong></p>'; } ?></th>
			</tr>
	</table>
	<p style="margin-top: 0; ">The plugin settings can be found <a href="<?php echo site_url(); ?>/wp-admin/admin.php?page=wc-settings&tab=checkout&section=ag_epdq_checkout">here</a>.</p>
</form>
<hr />
<?php

}




/*-----------------------------------------------------------------------------------*/
/*	GDPR new bits
/*-----------------------------------------------------------------------------------*/

// Adding what we store in custom fields to Woo hook before removing of data.
function remove_epdq_direct_personal_data($order) {

	$epdq_meta_to_remove = apply_filters( 'woocommerce_privacy_remove_order_personal_data_meta', array(
			'Acceptance'  		=> 'text',
			'BRAND'      			=> 'text',
			'IP' 							=> 'text',
			'NCERROR'     		=> 'text',
			'PAYID'     			=> 'text',
			'PaymentMethod'   => 'text',
			'Status'     			=> 'text',
			'OrderAmount'     => 'text',
			'TRXDATE'     		=> 'text',
			'OrderCurrency'   => 'text',
			'OrderID'     		=> 'text',
			'_transaction_id' => 'text',
		) );

		if ( ! empty( $epdq_meta_to_remove ) && is_array( $epdq_meta_to_remove ) ) {
					foreach ( $epdq_meta_to_remove as $meta_keys => $data_types ) {
						$value = $order->get_meta( $meta_keys );
						if ( empty( $value ) || empty( $data_types ) ) {
							continue;
						}
						$anon_value = function_exists( 'wp_privacy_anonymize_data' ) ? wp_privacy_anonymize_data( $data_types, $value ) : '';

						$anon_value = apply_filters( 'woocommerce_privacy_remove_order_personal_data_meta_value', $anon_value, $meta_keys, $value, $data_types, $order );

						if ( $anon_value ) {
							$order->update_meta_data( $meta_keys, $anon_value );
						} else {
							$order->delete_meta_data( $meta_keys );
						}
					}
				}

}
add_action( 'woocommerce_privacy_before_remove_order_personal_data', 'remove_epdq_direct_personal_data' );


function plugin_privacy_content() {
	return
	'<p>' . __( 'In the WooCommerce payments section you should list Barclays ePDQ as a payment processor.<br />
<br />
We accept payments through Barclays ePDQ. When processing payments, some of your data will be passed to Barclays, including information required to process or support the payment, such as the purchase total and billing information.<br /><br />
Once an order has been placed some of the data sent back from Barclays is stored to support the payment and order, this data will remain part of the order until it is deleted.
<br/><br />
Please see Barclays ePDQ Privacy Policy for more details.' ) . '</p>';
}


function add_suggested_privacy_content() {
	$content = plugin_privacy_content();
	wp_add_privacy_policy_content( __( 'AG ePDQ Direct Gateway' ), $content );
}

add_action( 'admin_init', 'add_suggested_privacy_content', 20 );
